unit TestGpTextFile1;
{Purpose  - Test of GpTextFile.
 Status   - Copyright  2001 Cocolsoft Computer Solutions
            'http://www.cocolsoft.com.au'.
 Compiler - Delphi 4.03}
{2001 Jan 23 - Program commenced}
{2001 Feb  5 - completed}

{Thanks to Primoz Gabrijelcic for his help in getting this Demo working.
 Thanks to Ray Lischner for unit WideLabel. Please note the Copyright
   notice in that unit. 'http://www.tempest-sw.com'}

{Notes -
1) The file 'english.txt' is 33 bytes long.
   It contains the text "Hello, this is written in English".
   It is not Unicode
2) The file 'serbian.txt is 76 bytes long.
   (2 byte header + 74 bytes content)
   It contains the text (when translated into English)
     "Hello, this is written in Serbian."
   It is Unicode.
   Thanks to Primoz for this file.
}
{Notes for Testing -
1) to test error conditions change ourReadBufferSize from even to odd
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    TestSerbian: TButton;
    TestEnglish: TButton;
    Label4: TLabel;
    Image1: TImage;
    Label5: TLabel;
    Label6: TLabel;
    Label7: TLabel;
    Image2: TImage;
    StatusBar1: TStatusBar;
    procedure TestEnglishClick(Sender: TObject);
    procedure TestSerbianClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

USES
  GpTextFile,
  WideLabel,
  GPHugeF;
var
  directoryName : STRING;
  myWideLabel   : TWideLabel;

procedure TForm1.TestEnglishClick(Sender: TObject);
{Read and display the file English}
{Use as many functions and properties etc. as we can}
const
  ourReadBufferSize = 100; {arbitrary size but must be even}
  {ourReadBufferSize = 101;} {this will produce an Error}
var
  bytesReadIn      : LONGINT;
  GpTextFileStream : TGpTextFileStream;
  englishFileName  : STRING;
  ourReadBuffer    : WideString;
begin
  englishFileName := directoryName + 'English.TXT';
  GpTextFileStream := TGpTextFileStream.Create(englishFileName,
                                               accRead); {only want to read}
TRY

  ShowMessage('The file name is >' + GpTextFileStream.FileName +
             '<' + ' and size is ' +
             IntToStr(GpTextFileStream.Size) +
             ' bytes');
  if GpTextFileStream.Is16Bit then
    MessageDlg('The file is Unicode with 16 bit (UCS-2) encoding',
               mtInformation, [mbOK], 0)
  else begin
    if GpTextFileStream.IsUnicode then
      MessageDlg('The file is Unicode with 16 bit (UCS-2) encoding or 8 bit (UTF-8) encoding',
                 mtInformation, [mbOK], 0)
    else
      MessageDlg('The file is NOT Unicode', mtInformation, [mbOK], 0); {<== correct}
  end;
  MessageDlg('The CodePage is ' +
             IntToStr(GpTextFileStream.CodePage),
               mtInformation, [mbOK], 0);
  {Note - Read or Write may set property WindowsError.
  Even though we have not yet called Read or Write we will (for fun) call
  WindowsError}
  if GpTextFileStream.WindowsError = 0 then
    MessageDlg('No Windows error', mtInformation, [mbOK], 0) {<== correct}
  else
    MessageDlg('Windows ERROR'   , mtInformation, [mbOK], 0);
  {Now Read}
  SetLength(ourReadBuffer, ourReadBufferSize);
 TRY
  bytesReadIn := GpTextFileStream.Read(ourReadBuffer[1], ourReadBufferSize);
  MessageDlg('Bytes read in was ' +
             IntToStr(bytesReadIn) +
             ' bytes', mtInformation, [mbOK], 0);
  if GpTextFileStream.WindowsError = 0 then begin
    MessageDlg('No error in read', mtInformation, [mbOK], 0); {<== correct}

    myWideLabel.Top     := 130;     //by commenting this out you see
                                    //it appears as a blank at top left
    myWideLabel.Left    := 416;
    myWideLabel.Width   := 200;
    myWideLabel.Height  := 50;
    myWideLabel.Color        := clAqua;
    myWideLabel.Font.Color   := clBlack;

    SetLength(ourReadBuffer, bytesReadIn DIV SizeOf(WideChar));
    myWideLabel.Caption := ourReadBuffer;
    myWideLabel.Visible := True;
    ShowMessage(myWideLabel.Caption); {To prove something is in myWideLabel.
                                      converts wide to Ansi}
  end
  else
    MessageDlg('ERROR in read'   , mtInformation, [mbOK], 0);
 EXCEPT
   ON E: EGpTextFileStream DO BEGIN
     {We substitute our own message}
     MessageDlg('Got an EGpTextFileStream error'   , mtInformation, [mbOK], 0);
     {A different way to display the message from GpTextFile}
     ShowMessage(E.Message);
     {The following assumes the exception was raised using CreateFmtHelp}
     ShowMessage('The Help Context was ' + IntToStr(E.HelpContext));
   END;
   ON EGpHugeFile DO
     {We substitute our own message}
     ShowMessage('Got an EGpHughFile       error');
 END;{TRY}

FINALLY
  GpTextFileStream.Free;
END;{TRY}
end;

procedure TForm1.TestSerbianClick(Sender: TObject);
{Read and display the file Serbian}
{Use as many functions and properties etc. as we can}
const
  ourReadBufferSize = 100; {arbitrary size but must be even}
var
  bytesReadIn      : LONGINT;
  GpTextFileStream : TGpTextFileStream;
  serbianFileName  : STRING;
  ourReadBuffer    : WideString;
begin
  serbianFileName := directoryName + 'Serbian.TXT';
  GpTextFileStream := TGpTextFileStream.Create(serbianFileName,
                                               accRead); {only want to read}
TRY
  ShowMessage('The file name is >' + GpTextFileStream.FileName +
             '<' + ' and size is ' +
             IntToStr(GpTextFileStream.Size) +
             ' bytes');
  if GpTextFileStream.Is16Bit then
    ShowMessage('The file is Unicode with 16 bit (UCS-2) encoding'){<== correct}
  else begin
    if GpTextFileStream.IsUnicode then
      ShowMessage('The file is Unicode with 16 bit (UCS-2) encoding or 8 bit (UTF-8) encoding')
    else
      ShowMessage('The file is NOT Unicode');
  end;
  ShowMessage('The CodePage is ' +
             IntToStr(GpTextFileStream.CodePage));
  {Note - Read or Write may set property WindowsError.
  Even though we have not yet called Read or Write we will (for fun) call
  WindowsError}
  if GpTextFileStream.WindowsError = 0 then
    ShowMessage('No Windows error')                                {<== correct}
  else
    ShowMessage('Windows ERROR');
  {Now Read}
  SetLength(ourReadBuffer, ourReadBufferSize);
  bytesReadIn := GpTextFileStream.Read(ourReadBuffer[1], ourReadBufferSize);
 TRY
  ShowMessage('Bytes read in was ' +
             IntToStr(bytesReadIn) +
             ' bytes');
  if GpTextFileStream.WindowsError = 0 then begin
    ShowMessage('No error in read');                               {<== correct}

    myWideLabel.Top     := 250;    
    myWideLabel.Left    := 416;
    myWideLabel.Width   := 200;
    myWideLabel.Height  := 50;
    myWideLabel.Color        := clAqua;
    myWideLabel.Font.Color   := clBlack;

    SetLength(ourReadBuffer, bytesReadIn DIV SizeOf(WideChar));
    myWideLabel.Caption := ourReadBuffer;
    myWideLabel.Visible := True;
    ShowMessage(myWideLabel.Caption); {To prove something is in myWideLabel.
                                      converts wide to Ansi}

  end
  else
    ShowMessage('ERROR in read');
 EXCEPT
   ON EGpTextFileStream DO
     ShowMessage('Got an EGpTextFileStream error');
   ON EGpHugeFile DO
     ShowMessage('Got an EGpHughFile       error');
 END;{TRY}

FINALLY
  GpTextFileStream.Free;
END;{TRY}

end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  myWideLabel := TWideLabel.Create(Form1);
  myWideLabel.Visible := False;
  myWideLabel.Parent  := Form1;
end;

INITIALIZATION
  directoryName:=SysUtils.ExtractFilePath(Application.Exename); {not foolproof}

end.
